<?php
/**
 * @version		1.0
 * @package		Joomla
 * @subpackage	OS Membership
 * @author  Denis Moskvin
 * @copyright	Copyright (C) 2016 PayAnyWay
 * @license		GNU/GPL
 */
// no direct access
defined( '_JEXEC' ) or die ;

define( 'PAW_TRANSACTION_OK',       0 );
define( 'PAW_TRANSACTION_FAILED',   1 );
define( 'PAW_TRANSACTION_UNKNOWN',  2 );


/**
 * PAW payment class
 *
 */
class os_paw extends os_payment {

    /**
     * is mode test? (false = prod)
     *
     * @var boolean
     */
    var $_mode = 0;

    /**
     * Assistant url
     *
     * @var string
     */
    var $_url = null;

    /**
     * Array of params will be posted to server
     *
     * @var string
     */
    var $_params = array();

    /**
     * Array containing data posted from payanyway
     *
     * @var array
     */
    var $_data = array();

    /***********************************************************************
     *** Class Constructor                                               ***
     ***********************************************************************/
    function os_paw($params) {
        parent::setName('os_paw');
        parent::os_payment();
        parent::setCreditCard(false);
        parent::setCardType(false);
        parent::setCardCvv(false);
        parent::setCardHolderName(false);
        $this->_mode = $params->get('paw_mode') ;
        if ($this->_mode) {
            $this->setParam('MNT_TEST_MODE', '1');
        }
        else {
            $this->setParam('MNT_TEST_MODE', '0');
        }

        // testing url
        // $this->_url = 'https://demo.moneta.ru/assistant.htm';
        $this->_url = 'https://www.payanyway.ru/assistant.htm';

        $this->setParam('MNT_ID', $params->get('paw_account_id'));
        $this->setParam('MNT_ACCOUNT_CODE', $params->get('paw_account_code'));
        $this->setParam('MNT_CURRENCY_CODE', $params->get('paw_currency'));
    }

    function getError()
    {
        if( $this->myError != 0 ) {
            // Internal Error
            return $this->myError;
        } else {
            // payanyway Error
            if( $this->getTrxnStatus() == 'True' ) {
                return PAW_TRANSACTION_OK;
            } elseif( $this->getTrxnStatus() == 'False' ) {
                return PAW_TRANSACTION_FAILED;
            } else {
                return PAW_TRANSACTION_UNKNOWN;
            }
        }
    }

    function getErrorMessage()
    {
        if( $this->myError != 0 ) {
            // Internal Error
            return $this->myErrorMessage;
        } else {
            // payanyway Error
            return $this->getTrxnError();
        }
    }

    /**
     * Set param value
     *
     * @param string $name
     * @param string $val
     */
    function setParam($name, $val) {
        $this->_params[$name] = $val;
    }

    function getParam($name) {
        $result = null;
        if (isset($this->_params[$name])) {
            $result = $this->_params[$name];
        }
        return $result;
    }

    /**
     * Setup payment parameter
     *
     * @param array $params
     */
    function setParams($params) {
        foreach ($params as $key => $value) {
            $this->_params[$key] = $value ;
        }
    }

    /**
     * Submit post to paypal server
     *
     */
    function submitPost() {
        ?>
        <div class="contentheading">Processing ...</div>
        <form method="post" action="<?php echo $this->_url; ?>" name="osm_form" id="osm_form">
            <?php
            foreach ($this->_params as $key => $val) {
                if ($key != 'MNT_ACCOUNT_CODE') {
                    echo '<input type="hidden" name="' . $key . '" value="' . $val . '" />';
                    echo "\n";
                }
            }
            ?>
            <script type="text/javascript">
                function redirect() {
                    document.osm_form.submit();
                }
                setTimeout('redirect()', 3000);
            </script>
        </form>
        <?php
    }


    function processPayment($row, $data) {
        $Itemid = JRequest::getInt('Itemid');
        $siteUrl = JURI::base() ;
        $amount = number_format($data['amount'], 2, '.', '');
        $transactionId = $Itemid . '_' . $row->id . '_' . time();

        // customer id
        $this->setParam('MNT_DESCRIPTION', 'User ID: ' . $row->id . ', email: ' . $row->email . ', Item ID: ' . $Itemid . ', Item name: ' . str_replace('"', '', $data['item_name']));
        $this->setParam('MNT_SUCCESS_URL', $siteUrl.'index.php?option=com_osmembership&view=complete&subscription_code='.$row->subscription_code.'&Itemid='.$Itemid);
        $this->setParam('MNT_FAIL_URL', $siteUrl.'index.php?option=com_osmembership&view=cancel&id='.$row->id.'&Itemid='.$Itemid);
        // Pay Url
        // $this->setParam('notify_url', $siteUrl.'index.php?option=com_osmembership&task=payment_confirm&payment_method=os_paw');

        $this->setParam('MNT_TRANSACTION_ID', $transactionId);
        $this->setParam('MNT_AMOUNT', $amount);

        if ($this->getParam('MNT_ACCOUNT_CODE')) {
            $signature = md5($this->getParam('MNT_ID') . $transactionId . $amount . $this->getParam('MNT_CURRENCY_CODE') . $this->getParam('MNT_TEST_MODE') . $this->getParam('MNT_ACCOUNT_CODE'));
            $this->setParam('MNT_SIGNATURE', $signature);
        }

        $this->submitPost();
    }


    function verifyPayment() {

        // get params
        $MNT_ID = $this->getRequestParameter('MNT_ID');
        $MNT_TRANSACTION_ID = $this->getRequestParameter('MNT_TRANSACTION_ID');
        $MNT_OPERATION_ID = $this->getRequestParameter('MNT_OPERATION_ID');
        $MNT_AMOUNT = $this->getRequestParameter('MNT_AMOUNT');
        $MNT_CURRENCY_CODE = $this->getRequestParameter('MNT_CURRENCY_CODE');
        $MNT_TEST_MODE = $this->getRequestParameter('MNT_TEST_MODE');
        $MNT_SIGNATURE = $this->getRequestParameter('MNT_SIGNATURE');

        // check signature if set
        if ($this->getParam('MNT_ACCOUNT_CODE')) {
            $signature = md5( $MNT_ID .  $MNT_TRANSACTION_ID . $MNT_OPERATION_ID . $MNT_AMOUNT . $MNT_CURRENCY_CODE . $MNT_TEST_MODE . $this->getParam('MNT_ACCOUNT_CODE'));
            if ($MNT_SIGNATURE != $signature) {
                echo 'FAIL';
                exit;
            }
        }

        $transactionData = explode('_', $MNT_TRANSACTION_ID);
        if (!isset($transactionData[1])) {
            echo 'FAIL';
            exit;
        }

        // enable membership
        $config = OSMembershipHelper::getConfig() ;
        $row = JTable::getInstance('OsMembership', 'Subscriber');
        $id = $transactionData[1];
        $transactionId = $MNT_OPERATION_ID;
        $amount = $MNT_AMOUNT;
        if ($amount < 0)
            return false ;

        $row->load($id);
        if ($row->published)
            return false ;

        $row->payment_date = date('Y-m-d H:i:s');
        $row->transaction_id = $transactionId ;
        $row->published = 1 ;
        $row->store();
        if($row->act == 'upgrade') {
            OSMembershipHelper::processUpgradeMembership($row);
        }

        JPluginHelper::importPlugin( 'osmembership' );
        $dispatcher = JDispatcher::getInstance();
        $dispatcher->trigger( 'onMembershipActive', array($row));
        OSMembershipHelper::sendEmails($row, $config);

        echo 'SUCCESS';
        exit;

    }


    function getRequestParameter($name)
    {
        $value = null;
        if (isset($_GET[$name])) {
            $value = $_GET[$name];
        }
        if (isset($_POST[$name])) {
            $value = $_POST[$name];
        }

        return $value;
    }

} 
?>